-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- For simplicity and SO that you can reproduce this example later, we
-- will use the pubs database!

-- First, you'll need to change it's recovery model
ALTER DATABASE pubs
	SET RECOVERY FULL
go

-- Next, you'll need to begin your "recovery strategy setup" by performing
-- some form of starting backup. In this case, we'll choose a full backup. 
-- NOTE: File or filegroup backups are also possible as a starting point
-- for recovery.
BACKUP DATABASE pubs
	TO DISK = N'C:\pubsbackup.bak'
	WITH INIT
go

-- Now we can begin to perform transactions as well as manage those transactions
-- through transaction log backups.

USE pubs
go

-- Again, for simplicity - we'll make changes to the titles table in order to 
-- setup the same set of transactions as in the slide: "Transaction Recovery & Checkpoints"

SELECT t.title_id 	AS [Title ID]
	, t.title		AS [Title]
FROM dbo.titles AS t
ORDER BY [Title ID]
go

-- This is transaction 1 from slide titled: "Transaction Recovery & Checkpoints"
UPDATE titles
	SET title = 'T1 commit b4 checkpoint - committed'
	WHERE title_id = 'BU1032'
go

-- This is transaction 3 from slide titled: "Transaction Recovery & Checkpoints"
BEGIN TRANSACTION
UPDATE titles
	SET title = 'T2 change b4 checkpoint - NOT committed'
	WHERE title_id = 'BU1111'

-- go to another window and run a checkpoint in this database

-- shutdown the server with nowait (power failure, etc.)

-- copy the files for resetting scenario

-- delete the log

-- restart server

-- check databases... pubs should be suspect and not listed!
sp_helpdb
go

-- can we get in?
USE pubs
go

-- ERROR 945

-- can we force it? yes! with emergency mode.
USE master
go

sp_configure 'allow updates', 1
go
reconfigure with override
go

update master.dbo.sysdatabases
	set status = 32768
	where name = 'pubs'
go

sp_configure 'allow updates', 0
go
reconfigure with override
go

-- Now can we get in?
USE pubs
go

-- Sure, but what's the state of the database?

SELECT t.title_id 	AS [Title ID]
	, t.title		AS [Title]
FROM dbo.titles AS t
ORDER BY [Title ID]
go

-- So - are you sure you can guarantee that all your data is consistent?
-- SQL Server can't - at least not without the log. This is why Microsoft
-- says that when a database is suspect you should begin your recovery process...

-- Restore the original version of the database back to the start:
RESTORE DATABASE pubs
	FROM DISK = N'C:\pubsbackup.bak'
	WITH RECOVERY, REPLACE 
go

-- Reset the Recovery Model back to simple
ALTER DATABASE pubs
	SET RECOVERY SIMPLE
go

-- If you want to completely re-install pubs from the sample scripts use
-- OSQL to run the install\instpubs.sql script. 
-- See the Books Online for more details.